//Software base para OPX001 (Full)
//Controlador ESP32
//Utilização da biblioteca base
//Chamada das IOs

//Inclusão da biblioteca base
#include <OPX001_IO.h>

//Inclusão biblioteca SPI (caso necessário)
#include <SPI.h>

//Definicao do numero de IOs (DI, AI, AT, DO, AO, E2P)
OPX001_IO OPX(18, 8, 4, 18, 4, 1);

//Pinos auxiliares
#define ENTX_pin 02 //Pino de comunicação RS485
#define SDA2 32
#define SCL2 33
#define MOSI 13
#define MISO 12
#define SCLK 14
#define CS 15

//Variaveis para controle das entradas e saidas 
bool Digital_In[18];
int Analog_In[8];
int Analog_Temp[4];
bool Digital_Out[18];
int Analog_Out[4];

void setup() {
  //Inicializa comunicações
  Wire1.begin(SDA2,SCL2,100000); // (i2C)
  Serial.begin(115200); // (USB)
  Serial1.begin(115200); // (TTL)
  Serial2.begin(115200); //(485)

  //Inicializa o OPX001
  OPX.setup();
}

void loop() {
  //************************************************************************************
  //Leitura das entradas digitais
  //Esse código deve rodar sempre para leitura das entradas digitais
  //Utilizar o Array Digital_In[0] a Digital_In[17] no software para controle
  //************************************************************************************
  for (int i=0; i<18; i++){
  Digital_In[i] = OPX.DI(i);  
  }
  
  //************************************************************************************
  //Leitura das entradas Analógicas
  //Esse código deve rodar sempre para leitura das entradas analógicas
  //Utilizar o Array Analog_In[0] a Array Analog_In[7] no software para controle
  //Elas tem o valor de 0-10000 equivalentes a tensão de entrada 0-10V
  //************************************************************************************
  for (int i=0; i<8; i++){
  Analog_In[i] = OPX.AI(i);  
  }

  //************************************************************************************
  //Leitura das entradas temperatura
  //Esse código deve rodar sempre para leitura das entradas de temperaturas
  //Utilizar o Array Analog_Temp[0] a Array Analog_Temp[3] no software para controle
  //Elas tem o valor de 0-4095. Devem ser calibradas para cada tipo de sensor
  //************************************************************************************
  for (int i=0; i<4; i++){
  Analog_Temp[i] = OPX.AT(i);  
  }

  //************************************************************************************
  //Acionamento das saídas digitais
  //Esse código deve rodar sempre para acionar as saídas digitais
  //Utilizar o Array Digital_Out[0] a Digital_Out[17] no software para controle
  //************************************************************************************
  for (int i=0; i<18; i++){
  OPX.DO(i,Digital_Out[i]);  
  }

  //************************************************************************************
  //Acionamento das saídas analógicas
  //Esse código deve rodar sempre para acionar as saídas analógicas
  //Utilizar o Array Analog_Out[0] a Analog_Out[3] no software para controle
  //Elas tem o valor de 0-10000 equivalentes a tensão de de saída 0-10V
  //************************************************************************************
  for (int i=0; i<4; i++){
  OPX.AO(i,Analog_Out[i]);  
  }

  //************************************************************************************
  //Leitura da memória
  //Esse código deve rodar sempre para leitura de um valor armazenado em uma posição de memória
  //A memória possui 65536 posições de memória (bytes)
  //************************************************************************************
  int Variavel = OPX.RE2P(200); //nesse caso leitura da posição de memória 200

  //************************************************************************************
  //Escrita na memória
  //Esse código deve rodar sempre para escrita de um valor em uma posição de memória
  //A memória possui 65536 posições de memória (bytes)
  //************************************************************************************
  OPX.WE2P(137, Variavel); //nesse caso escreve o valor da "Variável" na posição 137

  //A função poll realiza a atualização dos IOs através do barramento de comunicação
  //Idealmente deve-se chamar uma vez por ciclo de máquina no início ou fim da função Loop
  OPX.poll();
}
